;  ------------------------------------------------------------------

;  FULL-FEATURED WAIT RECORD SAMPLE SCRIPT.

;  This script shows how to do a WAIT RECORD script
;  incorporating all the record locking, duplicate key, and
;  function key processing described in the recent series of
;  articles.

;  This script controls data entry into the table TEST.DB, which
;  updates entries in the table LOOKUP3.DB as it goes along.
;  All processing is done in table-view format.

;  The WAIT RECORD in this script is appropriate for data entry
;  into a Paradox table in table-view format.  For form-view format,
;  you need to revise the list of UNTIL keys in the WAIT command.

;  ------------------------------------------------------------------

Clear

;  ------------------------------------------------------------------

;  ------------------ Basic flow of WAIT_LOOP() ---------------------

;     is_arriving = True

;     WHILE TRUE

;        If is_arriving then
;           is_modifiable = True or False, depending
;           If is_modifiable then
;              CopyToArray before_rec
;              Record_Arrival()
;           is_arriving = False

;        WAIT RECORD

;        Non_Move_Keys()
;        If wait_looping then ----------------------------> LOOP

;        If not is_deleting and not is_undoing 
;           and "KeyViol" then ---------------------------> LOOP

;        If not is_deleting and not is_undoing and is_modified then
;           CopyToArray after_rec
;           Key_Validate()
;              (If a new record is locked, its key is
;                 written into before_rec)
;           If Key_Validate fails then -------------------> LOOP

;        If is_modifiable then
;           Record_Departure()
;           If Record_Departure fails then ---------------> LOOP

;        Move_Keys()

;        If sysmode()="Main" then --------------------> QUITLOOP

;        If "KeyViol" then -------------------------------> LOOP

;        is_arrival = True

;     ENDWHILE

;  ------------------------------------------------------------------

Proc Wait_Loop()

;  Wait Record with all fixes for function key traps,
;  and the fix for duplicate keys.

Private 
   after_rec,        ;the record after the WAIT
   before_rec,       ;the record before the WAIT
   is_arriving,      ;whether we're arriving at a new record
   is_deleting,      ;whether we're about to delete a record
   is_modified,      ;whether the record has been modified
   is_readonly,      ;whether the record is in a read-only state
   is_undoing,       ;whether we're performing an UNDO (Ctrl-U)
   wait_key,         ;the "Until" key pressed in response to the WAIT
   wait_looping,     ;whether to loop back to the WAIT command again
   wait_msg          ;the error message to display on the WAIT command

;  Intialization

CoEditKey               ;everything will be done in co-edit mode
is_arriving = True      ;we begin by arriving at a new record
wait_msg = ""           ;no error message yet

;  This is the WAIT loop.  The WAIT command might be executed repeatedly.

While True

;  Processing for when we're arriving at a new record

   If is_arriving then           ;we just arrived at a new record
      is_arriving = False        ;arrival is being handled now

      If not recordstatus("New") and      ;it's an existing record
         not recordstatus("Locked") then  ;but not yet locked
         LockRecord                       ;attempt to lock the record
         If not retval then               ;the lock failed
            wait_msg = errormessage()     ;display the error message
         Endif
      Endif

      If recordstatus("New") or        ;it's a new record, or
         recordstatus("Locked") then   ;it's a locked record
         is_readonly = False           ;let the user modify it
         CopyToArray before_rec        ;remember record, pre-change
         Record_Arrival()              ;perform the arrival proc
         If not retval then            ;error in arrival proc
            is_readonly = True         ;don't let user change record
            is_arriving = True         ;repeat arrival proc next loop
         Endif
      Else                             ;an existing record, not lockd
         is_readonly = True            ;don't let the user modify it
      Endif

      is_modified = recordstatus("Modified")    ;is rcd modified yet?
   Endif

   If is_readonly then           ;if user may not change the record
      ImageRights ReadOnly       ;prohibit changes
   Endif

;  The WAIT RECORD command:  Let the user view and edit this record

   Wait Record
      Prompt "Editing the "+table()+" table",
             "[F2]=Done Editing"
      Message wait_msg
      Until "Do_it!",                               ;termination keys
            "Up","Down","PgUp","PgDn","Home","End", ;movement keys
            "Ins","Del","Undo","Zoom","ZoomNext",   ;movement keys
            "Help","FieldView","F35",               ;non-move keys
            -33,                                    ;Alt-F
            -31,                                    ;Ctrl-S
            "Dos","DosBig","LockKey","KeyLookup",   ;illegal keys
            "GoKey","Rotate","F32"                  ;illegal keys

   wait_key = retval    ;remember which UNTIL key the user pressed
   wait_msg = ""        ;clear the previous error message
   ImageRights          ;restore full imagerights

   If recordstatus("Modified") then    ;did the user modify it?
      is_modified = True               ;yes; remember it's modified
   Endif

;  Process function keys which do not cause record movement

   wait_looping = True        ;assume we will loop right back to WAIT
   is_deleting  = False       ;the [Del] key might set this to True
   is_undoing   = False       ;the [Ctrl-U] key will set this to True

   Non_Move_Keys()            ;process the non-movement keys

   If wait_looping then Loop Endif     ;no further work needed

;  We are about to depart from the record, so perform the 
;  record-departure processing.

   If not is_deleting and not is_undoing then  ;not [Del] or [Ctrl-U]
      If recordstatus("KeyViol") then     ;we still have dup key
         Beep                             ;display an error message
         wait_msg = "Fix the duplicate key!"
         CtrlHome    ;move the cursor to where key field usually is
         Loop                             ;do not proceed
      Endif

      If is_modified then                 ;user changed the record
         CopyToArray after_rec            ;remember the new record
         Key_Validate()                   ;validate the key
         If not retval then Loop Endif    ;trouble with this key
      Endif
   Endif

   If not is_readonly then                ;record could be modified
      Record_Departure()                  ;perform the departure proc
      If not retval then Loop Endif       ;trouble with this record
   Endif
   is_arriving = True         ;next loop will be new record arrival

;  Process the record-departure function key

   Move_Keys()                ;process the keys

;  If we're out of CoEdit mode, because of a Do_it!, then quit

   If sysmode()="Main" then QuitLoop Endif

;  If we're in a key-violation status, display an error message

   If recordstatus("KeyViol") then
      Beep
      wait_msg = "Another record with this key already exists"
      CtrlHome    ;move the cursor to where the key field usually is
   Endif
EndWhile

EndProc

;  ------------------------------------------------------------------

Proc Non_Move_Keys()

;  Processes the keys which do not cause record movement

Switch
   Case wait_key="Help":
      If is_readonly then     ;ImageRights is "ReadOnly"
         Beep                 ;so [F1] is illegal
      Else                    ;the user may change the record
         Lookup_Echo()        ;perform the [F1] lookup process
      Endif

   Case wait_key="Del":
      If is_readonly then           ;ImageRights is "ReadOnly"
         Beep                       ;[Del] is not allowed
      Else                          ;the user can modify the record
         Wait_Del()                 ;display a yes/no confirming menu
         is_deleting = retval       ;whether we're to delete record
      Endif
      wait_looping = not is_deleting   ;whether we're all done

   Case wait_key="FieldView" or wait_key="F35":
      Wait_FieldView()

   Case wait_key="Dos"     or wait_key="DosBig" or
        wait_key="LockKey" or wait_key="KeyLookup" or
        wait_key="GoKey"   or wait_key="Rotate" or
        wait_key="F32":
            Beep

   Case wait_key="Undo":
      If is_readonly then           ;ImageRights is "ReadOnly"
         Beep                       ;[Ctrl-U] is not allowed
      Else                          ;the user can modify the record
         is_undoing = True          ;remember we're undoing
         Switch
            Case recordstatus("Locked"):     ;normal locked record
               Undo                          ;perform the Ctrl-U
            Case recordstatus("Modified"):   ;normal locked record
               Undo                          ;perform the Ctrl-U
            Case recordstatus("New"):     ;a new blank record
               is_deleting = True         ;do a [Del], not [Ctrl-U]
         EndSwitch      ;otherwise, don't do [Ctrl-U] at all
         CopyToArray after_rec      ;remember the undone record
         wait_looping = False       ;proceed with depart-rcd proc
      Endif

   Case wait_key=-33:         ;Alt-F:  Fill in a field
      [Field 4] = substr(strval(rand()),3,5)

   Case wait_key=-31:         ;Alt-S:  Display record status
      wait_msg =    "New="      + strval(recordstatus("New")) +
                 "   Locked="   + strval(recordstatus("Locked")) +
                 "   Modified=" + strval(recordstatus("Modified")) +
                 "   KeyViol="  + strval(recordstatus("KeyViol"))

   Otherwise:
      wait_looping = False
EndSwitch

If recordstatus("Modified") then    ;did the function modify the rcd?
   is_modified = True               ;yes; remember it's modified
Endif

EndProc

;  ------------------------------------------------------------------

Proc Move_Keys()

;  Processes the keys which cause record movement

Switch
   Case wait_key="Do_it!":
      Do_it!

   Case wait_key="Zoom":
      Zoom_Echo(True)
      wait_msg = retval

   Case wait_key="Del":
      Del 
      If not recordstatus("Locked") then
         wait_msg = "Record Deleted."
      Else
         Beep
         wait_msg = "Unable to delete the record"
      Endif

   Case wait_key="Undo":
      If is_deleting then        ;undoing a new blank record
         Del                     ;we'll delete it instead
                                 ;(otherwise Ctrl-U would undo
                                 ;the prior record)
      Endif

   Otherwise:
      KeyPress wait_key
EndSwitch

If not isblank(window()) and isblank(wait_msg) then
   wait_msg = window()
Endif

EndProc

;  ------------------------------------------------------------------

Proc Record_Arrival()

;  Processing to be performed when you arrive at a new record

;  A special consideration:  You might want to test the value of
;  RECORDSTATUS("KeyViol").  If it is TRUE, then we are stuck in a 
;  duplicate key record for which the record-departure procedure has
;  already been performed.  In some applications, the record-arrival
;  procedure might need to undo what the work performed by the
;  record-departure procedure.

Private 
   field_3,    ;contents of Field 3
   x           ;keyboard character

Style Attribute 47
@ 0,0 ?? spaces(160)
@ 0,0 ?? "Your record-arrival procedure would be called here."
;  @ 1,0 ?? "Press any key to continue..."
Style
x = 32
;  x = getchar()
@ 1,0 Clear Eol
@ 0,0 Clear Eol

;  Locate and lock the corresponding record in LOOKUP3.

field_3 = before_rec["Field 3"]      
If not isblank(field_3) then
   MoveTo "Lookup3"
   Locate_Lookup3(field_3)
   MoveTo "Test"
   If not retval then Return False Endif
Endif

Return True

EndProc

;  ------------------------------------------------------------------

Proc Record_Departure()

;  Processing to be performed when you arrive at a new record

;  This procedure should return:
;     retval=True, if the record is okay to depart from
;     retval=False, if the record should not be departed from

;  Upon entry to this procedure, typically:
;     before_rec is an array of the record before the user made
;                 any changes to it;
;     after_rec  is an array of the record as it appears right now.
;     The record is locked.
;     The record is "modifiable" -- i.e., ImageRights is normal,
;        and it was modifiable for the user during the WAIT command.

;  Note the following special situations:

;  If we are processing a [Del] function, after_rec does not exist
;     or contains unreliable information.
;  If we are processing a [Ctrl-U] function, the [Ctrl-U] has already
;     been performed.  After_rec contains the record as is is right
;     now (i.e., undone).  THE RECORD IS NO LONGER LOCKED.
;  If the record has never been modified (is_modified=False),
;     after_rec does not exist or contains unreliable information.
;  If the record is a new unposted record, then before_rec will
;     exist but it should be entirely blank values.
;  If the record is a new posted record, then before_rec will be
;     entirely blank except for the key fields, which will contain
;     the key values with which the record has been posted.

Private x, old_field_3, new_field_3, old_count_3, new_count_3, 
   failure_point, msg

Style Attribute 79
@ 0,0 ?? spaces(160)
@ 0,0 ?? "Your record-departure procedure would be called here."
If is_deleting then
   ?? " *Deleting*"
Endif
;  @ 1,0 ?? "Press any key to continue.  Press [Esc] for departure error."
Style
x = 32
;  x = getchar()
@ 1,0 Clear Eol
@ 0,0 Clear Eol
If x=27 then
   wait_msg = "The record-departure procedure encountered a problem"
   Return False
Endif

;  Make sure the required fields are filled in

If not is_deleting and not is_undoing and is_modified and 
   isblank([Field 4]) then
      MoveTo [Field 4]
      Beep
      wait_msg = "This field must be filled in"
      Return False
Endif

;  Update the quantities in the LOOKUP3 table

failure_point = 0
old_field_3 = before_rec["Field 3"]
old_count_3 = before_rec["Field 3 Count"]
Switch
   Case is_deleting:
      new_field_3 = ""
      new_count_3 = 0
   Case not is_modified:
      new_field_3 = old_field_3
      new_count_3 = old_count_3
   Otherwise:
      new_field_3 = after_rec["Field 3"]
      new_count_3 = after_rec["Field 3 Count"]
EndSwitch
If not isblank(old_field_3) or not isblank(new_field_3) then
   MoveTo "Lookup3"
   If not isblank(old_field_3) then
      Locate_Lookup3(old_field_3)
      If not retval then
         failure_point = 1
      Else
         [Field 3 Count] = [Field 3 Count] - old_count_3
      Endif
   Endif
   If failure_point=0 and not isblank(new_field_3) then
      Locate_Lookup3(new_field_3)
      If not retval then
         failure_point = 2
      Else
         [Field 3 Count] = [Field 3 Count] + new_count_3
      Endif
   Endif
   If failure_point<>0 then
      msg = wait_msg
      If failure_point=2 and not isblank(old_field_3) then
         Locate_Lookup3(old_field_3)
         If not retval then
            failure_point = 3
         Else
            [Field 3 Count] = [Field 3 Count] + old_count_3
         Endif
      Endif
   Endif

   If failure_point<>0 then
      MoveTo "Test"
      Undo
      If failure_point=3 then [Field 3 Count]=0 Endif
      MoveTo "Lookup3"
   Endif

   UnlockRecord
   MoveTo "Test"

   If failure_point<>0 then
      Return False
   Endif
Endif

Return True

EndProc

;  ------------------------------------------------------------------

Proc Locate_Lookup3(v)

;  Locate and lock a record in the LOOKUP3 table

Private msg

msg = ""
If []<>v then
   Locate v
   If not retval then
      msg = "Record for "+v+" not found in "+table()
   Endif
Else
   retval = True
Endif

If retval and not recordstatus("Locked") then
   LockRecord
   If not retval then
      msg = "Could not lock the "+table()+" record -- "+errormessage()
   Else
      If []<>v then
         msg = "Record for "+v+" has been deleted from "+table()
         Unlockrecord
      Endif
   Endif
Endif

If not isblank(msg) then
   wait_msg = msg
   Beep
   Return False
Endif

Return True

EndProc

;  ------------------------------------------------------------------

Proc Key_Validate()

;  Validates the key field of the record being entered.

;  It tests:
;     1.  That all the key fields have been filled in
;            (except for fields the cursor cannot move to).
;     2.  That the key is unique.

;  Returns retval=True  if the key is okay,
;       or retval=False if it is not

;  If this is a new record, this procedure if it succeeds
;  guarantees that the key is valid now and will continue
;  to be valid when you depart from the record, because the
;  procedure posts the record and locks it.

;  If this is not a new record but you have changed the key,
;  this procedure tests whether the new key is unique right
;  at this moment.  However, it cannot guarantee that the
;  key will still be unique later when the record is unlocked.

;  This is an optional procedure.  It is not necessary for duplicate
;  record processing, but it can make your life easier for some
;  applications because it catches most (but not all) duplicate key
;  problems before the record-departure processing is performed.

;  Returns retval=True  if the key is okay,
;       or retval=False if it is not

Private
   is_field_modified,   ;whether a field has been modified
   is_key_modified,     ;whether the key has been modified
   is_key_dup,          ;whether the key is a duplicate
   is_new,              ;whether it's a new record
   n_key_fields,        ;# of key fields in the record
   i                    ;field counter

;  Initialization

is_new = recordstatus("New")     ;whether it's a new record

;  Make sure all the key fields have been filled in,
;  and see whether the key has been changed.

n_key_fields = nkeyfields(table())     ;# of key fields
is_key_modified = False                ;assume the key hasn't changed
For i from 1 to n_key_fields           ;for each key field
   is_field_modified =                 ;has the field been changed?
      (after_rec[i+1]<>before_rec[i+1])
   is_key_modified =                   ;has the key been changed?
      is_key_modified or is_field_modified

   If not isblank(after_rec[i+1]) then ;it's filled in -- okay
      Loop                             ;so try the next one
   Endif
   If not is_new and                ;it's an old record in which...
      not is_field_modified then    ;...this field was blank before
      Loop                          ;...so let it remain blank
   Endif

   MoveTo_Field_Num(i)        ;move to the field
   If not retval then         ;the key is not on the form
      Loop                    ;so accept it as is
   Endif
   Beep                                      ;indicate trouble
   wait_msg = "You must fill in this field"  ;the error message
   Return False                              ;look no further
EndFor

;  For a new record, locking the record validates the key

is_key_dup = False                  ;assume it's not a duplicate key
If not recordstatus("Locked") then  ;it's a new record
   LockRecord                       ;attempt to lock it
   If retval then                   ;the lock succeeded
      For i from 1 to n_key_fields  ;for each key field
         before_rec[i+1] = after_rec[i+1]    ;xfer the key fields
      EndFor
   Else                             ;the lock failed
      Beep                          ;indicate trouble
      If errorcode()=53 then        ;duplicate key error
         wait_msg = "Another record with this key already exists"
      Else
         wait_msg = errormessage()  ;some other locking error
      Endif
      is_key_dup = True             ;it's a duplicate
   Endif

;  For an existing record, we can check for a duplicate using the
;  KEYLOOKUP command, but we can't lock the new key because the
;  record is already locked.

Else If is_key_modified then  ;new key for an old record
   KeyLookup   ;sets errorcode()=0 for duplicate, =56 for non-dup.
   If errorcode()=0 then   ;KeyLookup succeeded, so it's a dup. key
      Beep                 ;indicate trouble
      wait_msg = "Another record with this key already exists"
      KeyLookup            ;restore the original record
      is_key_dup = True    ;it's a duplicate
   Endif
Endif Endif

;  On a duplicate key, move the cursor to the last key field
;  present on the form.

If is_key_dup then                        ;it's a duplicate key
   For i from n_key_fields to 1 step -1   ;visit keys backwards
      MoveTo_Field_Num(i)                 ;move cursor to the field
      If retval then QuitLoop Endif       ;successful move
   EndFor                                 ;try the previous field
   Return False                           ;return failure flag
Endif

Return True       ;the key is okay

EndProc

;  ------------------------------------------------------------------

Proc MoveTo_Field_Num(n)

;  Moves to the n-th field in a table.

;  Returns retval=True if successful, retval=False if not.

Private
   f,                ;the field name
   is_present,       ;whether the key field is present on the form
   orig_errorproc    ;original error procedure name

;  Determine the field name

f = Field_Name(n)
If isblank(f) then Return False Endif  ;invalid field number

;  Define a small error procedure to tell us when a field we try
;  to move to is not present on the form.

Proc MoveTo_Field_Num_Err()   ;special error procedure
   is_present = False         ;we were unable to move to that field
   Return 1                   ;proceed with the calling procedure
EndProc

;  Establish the new error procedure

If isassigned(errorproc) then    ;there is an error procedure defined
   orig_errorproc = errorproc    ;save its name for later
Endif
errorproc = "MoveTo_Field_Num_Err"  ;set up the new error procedure

;  Move to the field

is_present = True       ;assume we will be able to move to it
MoveTo Field f          ;move to the field, by name

;  Restore the original error procedure

errorproc = orig_errorproc
Release Procs MoveTo_Field_Num_Err

;  Return TRUE or FALSE, whether this procedure has succeeded

Return is_present

EndProc

;  ------------------------------------------------------------------

Proc Field_Name(n)

;  A function that returns the name of the n-th field in the
;  current table.

;  Since this capability is not directly available as a PAL function,
;  this procedure moves to the n-th field by issuing the menu
;  sequence {Image} {Zoom} {Field} and then counting fields from
;  the menu.

Private
   f,          ;the field name
   i           ;field counter

;  Test for invalid values

If n<0 then Return "" Endif         ;negative
If n=0 then Return "#" Endif        ;the record number column
If n>nfields(table()) then Return "" Endif   ;not that many fields

;  Get the field name from the Image/Zoom/Field menu

Menu {Image} {Zoom} {Field}   ;this shows a menu of all field names
For i from 2 to n             ;until the n-th field is reached
   Right                      ;move thru the field names in the menu
EndFor
f = menuchoice()              ;read the field name from the menu
Menu Esc                      ;get rid of the menu

Return f                      ;return the field name

EndProc

;  ------------------------------------------------------------------

;  Procedure to simulate Ctrl-Z using ECHO

Proc Zoom_Echo(echo_off_after)

;  Parameter echo_off_after -- True or False; whether to turn
;                              "Echo" off at the conclusion of
;                              this procedure

   Private k, w
   Zoom                 ;press [Ctrl-Z] for the user
   While True           ;loop for each keystroke the user presses
      If menuchoice()="Error" then  ;no more Ctrl-Z prompt, probably
         w = window()         ;capture error message, if any
         Echo Off             ;lower canvas to do SyncCursor
         SyncCursor           ;move canvas cursor to workspace
         Echo Normal          ;turn echo back off
         If row()<>0 then     ;make sure we're no longer on top row
            QuitLoop          ;which means we're all done zooming
         Endif
      Endif
      Echo Normal             ;show the user everything that happens
      k = getchar()                 ;let the user press a key
      If (-68<=k and k<=-59)        ;trap [F1] thru [F10]
         or (-113<=k and k<=-84 and k<>-108) ;trap other fcn keys
         or k=15 or k=-24 then      ;trap [Ctrl-O] and [Alt-O]
            Beep                    ;beep on a bad keypress
      Else                          ;it's an okay key
         KeyPress k                 ;so let the user press it
      Endif
   EndWhile                   ;do the next key
   If echo_off_after then     ;we're supposed to stop echoing
      Echo Off Endif          ;so turn off echo
   Return w                   ;all done; return final message
EndProc

;  ------------------------------------------------------------------

;  Procedure to simulate F1-Lookup-Help using ECHO

Proc Lookup_Echo()
   Private k
   Help                 ;press [F1] for the user
   If helpmode()<>"LookupHelp" then        ;no, it's not lookup help
      If helpmode()="Help" then            ;it's regular help ...
         Esc Endif                         ;... so escape from it
      Beep                 ;user may not press [F1] on this field
      Return               ;all done with help
   Endif
   Echo Normal                 ;show the user everything that happens
   While helpmode()<>"None"    ;while we're still in look-up help
      k = getchar()            ;let the user press a key
      Switch                   ;determine what kind of key it is
         Case k=15 or k=-24 or k=-59:   ;[Ctrl-O], [Alt-O], or [F1]
            Beep               ;these three keys are illegal
         Case k=26:            ;[Ctrl-Z]
            Zoom_Echo(False)   ;control the "zoom" process
         Otherwise:            ;any other key is okay ...
            KeyPress k         ;... so press it
      EndSwitch
   EndWhile
   Echo Off                    ;drop the canvas again
EndProc

;  ------------------------------------------------------------------

Proc Wait_Del()

;  Displays a menu when the user presses the [Del] key during
;  a WAIT command.

;  Returns retval=True if the record is deleted 
;  and retval=False if it is not.

Private wait_menu_sel

If recordstatus("New") then      ;a new record
   Return True                   ;needs no confirming
Endif

Beep

Message "Do you really want to delete this record?"
ShowMenu
   "No":       "Do not delete this record",
   "Yes":      "Go ahead and delete this record"
   to wait_menu_sel

Message ""
@ 1,0 Clear Eol
@ 0,0 Clear Eol

If wait_menu_sel<>"Yes" then Return False Endif

Return True

EndProc

;  ------------------------------------------------------------------

Proc Wait_FieldView()

;  Performs the "field-view" function during a WAIT command.

FieldView

Wait Field 
   Prompt "Field View",
          "Make changes to this field and press [Enter] to continue"
   Until "Enter", "Do_it!", "CtrlBackspace",
         "Dos", "DosBig"

Switch
   Case retval="CtrlBackspace":                    KeyPress retval
   Case retval<>"Enter" and retval<>"Do_it!":      Beep
EndSwitch

EndProc

;  ------------------------------------------------------------------

Proc Test_End_Menu()

;  Menu to display before terminating the test

Private end_menu_sel

While True
   Clear
   ShowMenu
      "Quit":     "All done",
      "Again":    "Repeat the test again"
      to end_menu_sel
   Clear

   If end_menu_sel="Esc" then end_menu_sel="Again" Endif

   Return end_menu_sel
EndWhile

EndProc

;  ------------------------------------------------------------------

Proc Wait_Record_Line_Test()

;  Wait Record with all fixes for function key traps

Private 
   key_fields,       ;an array with the name of each key field
   key_presents,     ;an array:  for each key field, is it on the form?
   n_key_fields,     ;the # of key fields
   i,n               ;temporary counters

;  Intialization

Clear
Reset

View "Lookup3"
MoveTo [Field 3]
CoEdit "Test"
n = min(nrecords(table())-16,6)
If n>0 then
   Menu {Image} {TableSize}
   For i from 1 to n Up EndFor
   Home
Endif

While True

;  The WAIT loop

   Wait_Loop()

;  Done with the WAIT loop.  Ask what to do next.

   Test_End_Menu()
   If retval="Quit" then QuitLoop Endif
EndWhile

EndProc

;  ------------------------------------------------------------------

Wait_Record_Line_Test()

;  ------------------------------------------------------------------

